@@ -567,7 +567,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicod
         for(qint32 i = 0; (qint64)i < vSize; ++i)
         {
             //TODO: Phase this out. We should not be messing with these flags outside the parser.
-            m_normalData.m_v[i].setSkipable(m_lmppData.m_v[i].isPureComment());
+            m_normalData.m_v[i].setPureComment(m_lmppData.m_v[i].isPureComment());
             m_normalData.m_v[i].setSkipable(m_lmppData.m_v[i].isSkipable());
         }
     }
@@ -618,7 +618,7 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             return false;
         }
 
-        ts >> curChar;
+        curChar = ts.read(1).unicode()[0];
 
         quint32 firstNonwhite = 0;
         bool    foundNonWhite = false;
@@ -645,10 +645,9 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
             if(ts.atEnd())
                 break;
 
-            ts >> curChar;
+            curChar = ts.read(1).unicode()[0];
         }
 
-        ++lineCount;
 
         switch(curChar.unicode())
         {
@@ -668,7 +667,7 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
 
                     if(m_pBuf[lastOffset + j] == '\n')
                     {
-                        ts >> curChar;
+                        curChar = ts.read(1).unicode()[0];
                         vOrigDataLineEndStyle.push_back(eLineEndStyleDos);
                         lastOffset = ts.pos();
                         break;
@@ -683,15 +682,33 @@ bool SourceData::FileData::preprocess(QTextCodec* pEncoding, bool removeComments
         if(removeComments)
             parser->removeComment(line);
 
-        //kdiff3 internally uses only unix style endings for simplicity.
+        ++lineCount;
         m_v.push_back(LineData(m_unicodeBuf, lastOffset, line.length(), firstNonwhite, parser->isSkipable(), parser->isPureComment()));
-        m_unicodeBuf->append(line).append('\n');
+        //The last line may not have an EOL mark. In that case don't add one to our buffer.
+        m_unicodeBuf->append(line);
+        if(curChar == '\n' || curChar == '\r')
+        {
+            //kdiff3 internally uses only unix style endings for simplicity.
+            m_unicodeBuf->append('\n');
+        }
 
         lastOffset = m_unicodeBuf->length();
     }
+    /*
+        Process trailing new line as if there were a blank non-terminated line after it.
+        But do nothing to the data buffer since this a phantom line needed for internal purposes.
+    */
+    if(curChar == '\n' || curChar == '\r')
+    {
+        ++lineCount;
+
+        parser->processLine("");
+        m_v.push_back(LineData(m_unicodeBuf, lastOffset, 0, 0, parser->isSkipable(), parser->isPureComment()));
+    }
+    //Check if we have two identical offsets at the end. Indicates a bug in the read loop.
+    Q_ASSERT(m_v.size() < 2 || m_v[m_v.size() - 1].getOffset() != m_v[m_v.size() - 2].getOffset());
 
     m_v.push_back(LineData(m_unicodeBuf, lastOffset));
-    Q_ASSERT(m_v.size() < 2 || m_v[m_v.size() - 1].getOffset() != m_v[m_v.size() - 2].getOffset());
 
     m_bIsText = true;
 
@@ -775,14 +792,15 @@ QTextCodec* SourceData::detectEncoding(const char* buf, qint64 size, qint64& ski
         if(buf[0] == '\xEF' && buf[1] == '\xBB' && buf[2] == '\xBF')
         {
             skipBytes = 3;
-            return QTextCodec::codecForName("UTF-8");
+            //Custom codec.
+            return QTextCodec::codecForName("UTF-8-BOM");
         }
     }
     skipBytes = 0;
     QByteArray s;
     /*
         We don't need the whole file here just the header.
-]    */
+    */
     if(size <= 5000)
         s = QByteArray(buf, (int)size);
     else
