@@ -1,42 +1,31 @@
-/**
- * Copyright (C) 2003-2007 by Joachim Eibl <joachim.eibl at gmx.de>
- * Copyright (C) 2018 Michael Reeves reeves.87@gmail.com
+/*
+ * KDiff3 - Text Diff And Merge Tool
  * 
- * This file is part of KDiff3.
- * 
- * KDiff3 is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 2 of the License, or
- * (at your option) any later version.
- * 
- * KDiff3 is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- * 
- * You should have received a copy of the GNU General Public License
- * along with KDiff3.  If not, see <http://www.gnu.org/licenses/>.
- * 
- */
+ * SPDX-FileCopyrightText: 2002-2011 Joachim Eibl, joachim.eibl at gmx.de
+ * SPDX-FileCopyrightText: 2018-2020 Michael Reeves reeves.87@gmail.com
+ * SPDX-License-Identifier: GPL-2.0-or-later
+*/
 #include "Utils.h"
 
 #include "fileaccess.h"
 
+#include <KLocalizedString>
+
 #include <QString>
 #include <QStringList>
 #include <QHash>
 #include <QRegExp>
-#include <KLocalizedString>
+#include <QRegularExpression>
 
 /* Split the command line into arguments.
  * Normally split at white space separators except when quoting with " or '.
- * Backslash is treated as meta character within single quotes ' only.
+ * Backslash is treated as meta.
  * Detect parsing errors like unclosed quotes.
  * The first item in the list will be the command itself.
- * Returns the error reasor as string or an empty string on success.
+ * Returns the error reason as string or an empty string on success.
  * Eg. >"1" "2"<           => >1<, >2<
- * Eg. >'\'\\'<            => >'\<   backslash is a meta character between single quotes
- * Eg. > "\\" <            => >\\<   but not between double quotes
+ * Eg. >'\'\\'<            => >'\<   backslash is a meta character
+ * Eg. > "\\" <            => >\<
  * Eg. >"c:\sed" 's/a/\' /g'<  => >c:\sed<, >s/a/' /g<
  */
 QString Utils::getArguments(QString cmd, QString& program, QStringList& args)
@@ -108,15 +97,15 @@ bool Utils::wildcardMultiMatch(const QString& wildcard, const QString& testStrin
 {
     static QHash<QString, QRegExp> s_patternMap;
 
-    QStringList sl = wildcard.split(QChar(';'));
+    const QStringList regExpList = wildcard.split(QChar(';'));
 
-    for(QStringList::Iterator it = sl.begin(); it != sl.end(); ++it)
+    for(const QString& regExp : regExpList)
     {
-        QHash<QString, QRegExp>::iterator patIt = s_patternMap.find(*it);
+        QHash<QString, QRegExp>::iterator patIt = s_patternMap.find(regExp);
         if(patIt == s_patternMap.end())
         {
-            QRegExp pattern(*it, bCaseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive, QRegExp::Wildcard);
-            patIt = s_patternMap.insert(*it, pattern);
+            QRegExp pattern(regExp, bCaseSensitive ? Qt::CaseSensitive : Qt::CaseInsensitive, QRegExp::Wildcard);
+            patIt = s_patternMap.insert(regExp, pattern);
         }
 
         if(patIt.value().exactMatch(testString))
@@ -126,6 +115,55 @@ bool Utils::wildcardMultiMatch(const QString& wildcard, const QString& testStrin
     return false;
 }
 
+//TODO: Only used by calcTokenPos.
+bool Utils::isCTokenChar(QChar c)
+{
+    return (c == '_') ||
+           (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') ||
+           (c >= '0' && c <= '9');
+}
+
+
+//TODO: Needed? Only user of isCTokenChar.
+/// Calculate where a token starts and ends, given the x-position on screen.
+void Utils::calcTokenPos(const QString& s, int posOnScreen, int& pos1, int& pos2)
+{
+    int pos = std::max(0, posOnScreen);
+    if(pos >= (int)s.length())
+    {
+        pos1 = s.length();
+        pos2 = s.length();
+        return;
+    }
+
+    pos1 = pos;
+    pos2 = pos + 1;
+
+    if(isCTokenChar(s[pos1]))
+    {
+        while(pos1 >= 0 && isCTokenChar(s[pos1]))
+            --pos1;
+        ++pos1;
+
+        while(pos2 < (int)s.length() && isCTokenChar(s[pos2]))
+            ++pos2;
+    }
+}
+
+QString Utils::calcHistoryLead(const QString& s)
+{
+    // Return the start of the line until the first white char after the first non white char.
+    int i = s.indexOf(QRegularExpression("\\S"));
+    if(i == -1)
+        return QString("");
+    
+    i = s.indexOf(QRegularExpression("\\s"), i);
+    if(Q_UNLIKELY(i == -1))
+        return s;// Very unlikely
+    
+    return s.left(i);
+}
+
 /*
     QUrl::toLocalFile does some special handling for locally visable windows network drives.
     If QUrl::isLocal however it returns false we get an empty string back.
@@ -141,3 +179,4 @@ QString Utils::urlToString(const QUrl &url)
 
     return result;
 }
+
